@@ -1,4 +1,7 @@
 """Top-level package for PyGeoOGC."""
+import asyncio
+import sys
+
 from pkg_resources import DistributionNotFound, get_distribution
 
 from .exceptions import (
@@ -18,3 +21,6 @@ try:
 except DistributionNotFound:
     # package is not installed
     pass
+
+if sys.platform.startswith("win"):
+    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
