@@ -1,16 +1,17 @@
 """Base classes and function for REST, WMS, and WMF services."""
-from collections import defaultdict, namedtuple
+from collections import defaultdict
 from itertools import product
 from pathlib import Path
+from types import SimpleNamespace
 from typing import Any, Dict, List, Optional, Tuple, Union
 from warnings import warn
 
 import pyproj
 import shapely.ops as ops
 import yaml
-from orjson import JSONDecodeError
 from requests import Response
-from shapely.geometry import MultiPolygon, Polygon
+from shapely.geometry import MultiPoint, MultiPolygon, Point, Polygon
+from simplejson import JSONDecodeError
 
 from . import utils
 from .core import ArcGISRESTfulBase, WFSBase, WMSBase
@@ -48,32 +49,56 @@ class ArcGISRESTful(ArcGISRESTfulBase):
 
     def oids_bygeom(
         self,
-        geom: Union[Polygon, Tuple[float, float, float, float]],
+        geom: Union[
+            Polygon,
+            Point,
+            MultiPoint,
+            Tuple[float, float],
+            List[Tuple[float, float]],
+            Tuple[float, float, float, float],
+        ],
         geo_crs: str = DEF_CRS,
         sql_clause: str = "",
+        distance: Optional[int] = None,
     ) -> None:
         """Get feature IDs within a geometry that can be combined with a SQL where clause.
 
         Parameters
         ----------
-        geom : Polygon or tuple
-            A geometry (Polgon) or bounding box (tuple of length 4).
+        geom : Polygon, Point, MultiPoint, tuple, or list of tuples
+            A geometry (Polygon, Point, MultiPoint), tuple of length 2 (x, y),
+            a list of tuples of length 2 [(x, y), ...], or bounding box (tuple of length 4).
         geo_crs : str
-            The spatial reference of the input geometry, defaults to EPSG:4326
+            The spatial reference of the input geometry, defaults to EPSG:4326.
         sql_clause : str, optional
-            A valid SQL 92 WHERE clause, default to an empty string i.e., no
+            A valid SQL 92 WHERE clause, default to an empty string.
+        distance : int, optional
+            The buffer distance for the input geometries in meters, default to None.
         """
         if not isinstance(sql_clause, str):
             raise InvalidInputType("sql_clause", str)
 
-        if isinstance(geom, tuple):
+        if isinstance(geom, tuple) and len(geom) == 2:
+            geom = Point(geom)
+        elif isinstance(geom, list) and all(len(g) == 2 for g in geom):
+            geom = MultiPoint(geom)
+
+        geom_query = None
+        if isinstance(geom, tuple) and len(geom) == 4:
             geom = MatchCRS.bounds(geom, geo_crs, self.crs)  # type: ignore
             geom_query = utils.ESRIGeomQuery(geom, self.out_sr).bbox()
+        elif isinstance(geom, Point):
+            geom = MatchCRS.geometry(geom, geo_crs, self.crs)
+            geom_query = utils.ESRIGeomQuery((geom.x, geom.y), self.out_sr).point()
+        elif isinstance(geom, MultiPoint):
+            geom = MatchCRS.geometry(geom, geo_crs, self.crs)
+            geom_query = utils.ESRIGeomQuery([(g.x, g.y) for g in geom], self.out_sr).multipoint()
         elif isinstance(geom, Polygon):
             geom = MatchCRS.geometry(geom, geo_crs, self.crs)
             geom_query = utils.ESRIGeomQuery(geom, self.out_sr).polygon()
-        else:
-            raise InvalidInputType("geom", "tuple or Polgon")
+
+        if geom_query is None:
+            raise InvalidInputType("geom", "Polygon, Point, MultiPoint, tuple, or list of tuples")
 
         payload = {
             **geom_query,  # type: ignore
@@ -82,6 +107,9 @@ class ArcGISRESTful(ArcGISRESTfulBase):
             "returnIdsOnly": "true",
             "f": self.outformat,
         }
+        if distance:
+            payload["distance"] = f"{distance}"
+            payload["units"] = "esriSRUnit_Meter"
 
         if len(sql_clause) > 0:
             payload.update({"where": sql_clause})
@@ -89,11 +117,14 @@ class ArcGISRESTful(ArcGISRESTfulBase):
         resp = self.session.post(f"{self.base_url}/query", payload)
 
         try:
-            self.featureids = resp.json()["objectIds"]
+            r_json = resp.json()["objectIds"]
+            if r_json is None:
+                raise ZeroMatched(self._zeromatched)
+            self.featureids = r_json
         except (KeyError, TypeError, IndexError, JSONDecodeError):
             raise ZeroMatched(self._zeromatched)
 
-    def oids_byfield(self, field: str, ids: Union[str, List[str]], return_m: bool = False) -> None:
+    def oids_byfield(self, field: str, ids: Union[str, List[str]]) -> None:
         """Get Object IDs based on a list of field IDs.
 
         Parameters
@@ -102,8 +133,6 @@ class ArcGISRESTful(ArcGISRESTfulBase):
             Name of the target field that IDs belong to.
         ids : str or list
             A list of target ID(s).
-        return_m : bool
-            Whether to activate the Return M (measure) in the request, defaults to False.
 
         Returns
         -------
@@ -147,7 +176,10 @@ class ArcGISRESTful(ArcGISRESTfulBase):
         resp = self.session.post(f"{self.base_url}/query", payload)
 
         try:
-            self.featureids = resp.json()["objectIds"]
+            r_json = resp.json()["objectIds"]
+            if r_json is None:
+                raise ZeroMatched(self._zeromatched)
+            self.featureids = r_json
         except (KeyError, TypeError, IndexError, JSONDecodeError):
             raise ZeroMatched(self._zeromatched)
 
@@ -175,10 +207,10 @@ class ArcGISRESTful(ArcGISRESTfulBase):
             "f": self.outformat,
         }
 
-        def getter(ids: Tuple[str, ...]) -> Union[Response, Tuple[str, ...]]:
+        def getter(ids: Tuple[str, ...]) -> Union[Dict[str, Any], Tuple[str, ...]]:
             payload.update({"objectIds": ", ".join(ids)})
             resp = self.session.post(f"{self.base_url}/query", payload)
-            r_json = resp.json()
+            r_json: Dict[str, Any] = resp.json()
             try:
                 if "error" in r_json:
                     return ids
@@ -315,7 +347,9 @@ class WMS(WMSBase):
 
         geographic_crs = pyproj.CRS.from_user_input(self.crs).is_geographic
 
-        def _getmap(args):
+        def _getmap(
+            args: Tuple[str, Tuple[Tuple[float, float, float, float], str, int, int]]
+        ) -> Tuple[str, bytes]:
             lyr, bnds = args
             _bbox, counter, _width, _height = bnds
 
@@ -323,8 +357,8 @@ class WMS(WMSBase):
                 _bbox = (_bbox[1], _bbox[0], _bbox[3], _bbox[2])
 
             payload["bbox"] = f'{",".join(str(c) for c in _bbox)}'
-            payload["width"] = _width
-            payload["height"] = _height
+            payload["width"] = str(_width)
+            payload["height"] = str(_height)
             payload["layers"] = lyr
             resp = self.session.get(self.url, payload)
             return f"{lyr}_dd_{counter}", resp.content
@@ -447,7 +481,7 @@ class WFS(WFSBase):
             The geometric prediacte to use for requesting the data, defaults to
             INTERSECTS. Valid predicates are:
             EQUALS, DISJOINT, INTERSECTS, TOUCHES, CROSSES, WITHIN, CONTAINS,
-            OVERLAPS, RELATE, DWITHIN, BEYOND
+            OVERLAPS, RELATE, BEYOND
 
         Returns
         -------
@@ -475,7 +509,6 @@ class WFS(WFSBase):
             "CONTAINS",
             "OVERLAPS",
             "RELATE",
-            "DWITHIN",
             "BEYOND",
         ]
         if predicate not in valid_predicates:
@@ -542,6 +575,10 @@ class WFS(WFSBase):
         if not isinstance(cql_filter, str):
             raise InvalidInputType("cql_filter", "str")
 
+        valid_methods = ["GET", "POST"]
+        if method not in valid_methods:
+            raise InvalidInputValue("method", valid_methods)
+
         payload = {
             "service": "wfs",
             "version": self.version,
@@ -557,8 +594,6 @@ class WFS(WFSBase):
         elif method == "POST":
             headers = {"content-type": "application/x-www-form-urlencoded"}
             resp = self.session.post(self.url, payload, headers)
-        else:
-            raise InvalidInputValue("method", ["GET", "POST"])
 
         utils.check_response(resp)
 
@@ -568,26 +603,26 @@ class WFS(WFSBase):
 class ServiceURL:
     """Base URLs of the supported services."""
 
-    def __init__(self):
+    def __init__(self) -> None:
         fpath = Path(__file__).parent.joinpath("static/urls.yml")
         with open(fpath) as fp:
             self.urls = yaml.safe_load(fp)
 
-    def _make_nt(self, service):
-        return namedtuple(service, self.urls[service].keys())(*self.urls[service].values())
+    def _make_nt(self, service: str) -> SimpleNamespace:
+        return SimpleNamespace(**self.urls[service])
 
     @property
-    def restful(self):
+    def restful(self) -> SimpleNamespace:
         return self._make_nt("restful")
 
     @property
-    def wms(self):
+    def wms(self) -> SimpleNamespace:
         return self._make_nt("wms")
 
     @property
-    def wfs(self):
+    def wfs(self) -> SimpleNamespace:
         return self._make_nt("wfs")
 
     @property
-    def http(self):
+    def http(self) -> SimpleNamespace:
         return self._make_nt("http")
