@@ -3,7 +3,6 @@ import asyncio
 import socket
 from concurrent import futures
 from dataclasses import dataclass
-from itertools import zip_longest
 from typing import (
     Any,
     Callable,
@@ -19,6 +18,7 @@ from typing import (
 from unittest.mock import _patch, patch
 
 import aiohttp
+import cytoolz as tlz
 import nest_asyncio
 import numpy as np
 import orjson as json
@@ -27,8 +27,8 @@ from defusedxml import cElementTree as etree
 from requests import Response, Session
 from requests.adapters import HTTPAdapter
 from requests.exceptions import RequestException
-from shapely.geometry import LineString, MultiPolygon, Point, Polygon, box
-from shapely.ops import transform
+from shapely import ops
+from shapely.geometry import LineString, MultiPoint, MultiPolygon, Point, Polygon, box
 from urllib3 import Retry
 
 from .exceptions import InvalidInputType, InvalidInputValue, ThreadingException, ZeroMatched
@@ -67,14 +67,19 @@ class RetrySession:
         self.session = Session()
         self.retries = retries
 
-        r = Retry(
-            total=retries,
-            read=retries,
-            connect=retries,
-            backoff_factor=backoff_factor,
-            status_forcelist=status_to_retry,
-            method_whitelist=False,
-        )
+        retry_args = {
+            "total": retries,
+            "read": retries,
+            "connect": retries,
+            "backoff_factor": backoff_factor,
+            "status_forcelist": status_to_retry,
+        }
+        if hasattr(Retry.DEFAULT, "allowed_methods"):
+            retry_args.update({"allowed_methods": False})
+        else:
+            retry_args.update({"method_whitelist": False})
+
+        r = Retry(**retry_args)
         adapter = HTTPAdapter(max_retries=r)
         for prefix in prefixes:
             self.session.mount(prefix, adapter)
@@ -109,7 +114,22 @@ class RetrySession:
         """Disable IPv6 and only use IPv4."""
         orig_getaddrinfo = socket.getaddrinfo
 
-        def getaddrinfo_ipv4(host, port, family=socket.AF_INET, ptype=0, proto=0, flags=0):
+        def getaddrinfo_ipv4(
+            host: str,
+            port: str,
+            family: socket.AddressFamily = socket.AF_INET,
+            ptype: int = 0,
+            proto: int = 0,
+            flags: int = 0,
+        ) -> List[
+            Tuple[
+                socket.AddressFamily,
+                socket.SocketKind,
+                int,
+                str,
+                Union[Tuple[str, int], Tuple[str, int, int, int]],
+            ]
+        ]:
             return orig_getaddrinfo(
                 host=host,
                 port=port,
@@ -144,7 +164,7 @@ async def _request_binary(
         The retrieved response as binary
     """
     async with session_req(url, **payload) as response:
-        return await response.read()
+        return await response.read()  # type: ignore
 
 
 async def _request_json(
@@ -169,7 +189,7 @@ async def _request_json(
         The retrieved response as json
     """
     async with session_req(url, **payload) as response:
-        return await response.json()
+        return await response.json()  # type: ignore
 
 
 async def _request_text(
@@ -194,7 +214,7 @@ async def _request_text(
         The retrieved response as string
     """
     async with session_req(url, **payload) as response:
-        return await response.text()
+        return await response.text()  # type: ignore
 
 
 async def _async_session(
@@ -236,7 +256,7 @@ async def _async_session(
 
 
 def async_requests(
-    urls: Union[List[str], Dict[str, Optional[MutableMapping[str, Any]]]],
+    url_payload: List[Tuple[str, Optional[MutableMapping[str, Any]]]],
     read: str,
     request: str = "GET",
     max_workers: int = 8,
@@ -249,8 +269,8 @@ def async_requests(
 
     Parameters
     ----------
-    urls : list of str or dict of str and dict
-        A list of URLs or URLs with their payloads to be retrieved.
+    url_payload : list of tuples
+        A list of URLs and payloads as a tuple.
     read : str
         The method for returning the request; binary, json, and text.
     request : str, optional
@@ -263,19 +283,13 @@ def async_requests(
     list
         A list of responses
     """
-    if not isinstance(urls, list) and not isinstance(urls, dict):
-        raise InvalidInputType("urls", "list of urls or dict of urls and payloads")
-
-    url_payload = urls if isinstance(urls, dict) else {u: None for u in urls}
-    chunked_urls = list(zip_longest(*[iter(url_payload.items())] * max_workers))
-    chunked_urls[-1] = tuple(i for i in chunked_urls[-1] if i is not None)
+    chunked_urls = tlz.partition_all(max_workers, url_payload)
 
     results: List[Union[str, MutableMapping[str, Any], bytes]] = []
     for chunk in chunked_urls:
         loop = asyncio.get_event_loop()
         results.append(loop.run_until_complete(_async_session(chunk, read, request)))  # type: ignore
-        del loop
-    return [x for y in results for x in y]  # type: ignore
+    return list(tlz.concat(results))
 
 
 def threading(
@@ -314,7 +328,7 @@ def threading(
             itr = future_to_itr[future]
             try:
                 data.append(future.result())
-            except Exception as exc:
+            except Exception as exc:  # noqa: B902
                 raise ThreadingException(itr, exc)
     return data
 
@@ -339,7 +353,12 @@ def traverse_json(
         The items founds in the JSON
     """
 
-    def extract(obj, path, ind, arr):
+    def extract(
+        obj: Optional[Union[List[Any], Dict[str, Any]]],
+        path: Union[str, List[str]],
+        ind: int,
+        arr: List[Any],
+    ) -> List[Any]:
         key = path[ind]
         if ind + 1 < len(path):
             if isinstance(obj, dict):
@@ -386,7 +405,7 @@ class ESRIGeomQuery:
     Parameters
     ----------
     geometry : tuple or Polygon
-        The input geometry which can be a point (x, y),
+        The input geometry which can be a point (x, y), a list of points [(x, y), ...],
         bbox (xmin, ymin, xmax, ymax), or a Shapely's Polygon.
     wkid : int
         The Well-known ID (WKID) of the geometry's spatial reference e.g., for EPSG:4326,
@@ -395,22 +414,33 @@ class ESRIGeomQuery:
         for reference.
     """
 
-    geometry: Union[Tuple[float, float], Tuple[float, float, float, float], Polygon]
+    geometry: Union[
+        Tuple[float, float], List[Tuple[float, float]], Tuple[float, float, float, float], Polygon
+    ]
     wkid: int
 
     def point(self) -> Dict[str, Union[str, bytes]]:
         """Query for a point."""
-        if len(self.geometry) != 2:
+        if not isinstance(self.geometry, tuple) and len(self.geometry) != 2:
             raise InvalidInputType("geometry (point)", "tuple", "(x, y)")
 
         geo_type = "esriGeometryPoint"
         geo_json = dict(zip(("x", "y"), self.geometry))
         return self.get_payload(geo_type, geo_json)
 
+    def multipoint(self) -> Dict[str, Union[str, bytes]]:
+        """Query for a multi-point."""
+        if not isinstance(self.geometry, list) and any(len(g) != 2 for g in self.geometry):  # type: ignore
+            raise InvalidInputType("geometry (multi-point)", "list of tuples", "[(x, y), ...]")
+
+        geo_type = "esriGeometryMultipoint"
+        geo_json = {"points": [[x, y] for x, y in self.geometry]}  # type: ignore
+        return self.get_payload(geo_type, geo_json)
+
     def bbox(self) -> Dict[str, Union[str, bytes]]:
         """Query for a bbox."""
-        if len(self.geometry) != 4:
-            raise InvalidInputType("geometry (bbox)", "tuple", BOX_ORD)
+        if not isinstance(self.geometry, (tuple, list)) and len(self.geometry) != 4:
+            raise InvalidInputType("geometry (bbox)", "tuple or list", BOX_ORD)
 
         geo_type = "esriGeometryEnvelope"
         geo_json = dict(zip(("xmin", "ymin", "xmax", "ymax"), self.geometry))
@@ -448,12 +478,14 @@ class MatchCRS:
     """
 
     @staticmethod
-    def geometry(geom: Union[Polygon, MultiPolygon], in_crs: str, out_crs: str) -> Polygon:
-        if not isinstance(geom, (Polygon, MultiPolygon)):
-            raise InvalidInputType("geom", "Polygon")
+    def geometry(
+        geom: Union[Polygon, MultiPolygon, Point, MultiPoint], in_crs: str, out_crs: str
+    ) -> Union[Polygon, MultiPolygon, Point, MultiPoint]:
+        if not isinstance(geom, (Polygon, MultiPolygon, Point, MultiPoint)):
+            raise InvalidInputType("geom", "Polygon, MultiPolygon, Point, or MultiPoint")
 
         project = pyproj.Transformer.from_crs(in_crs, out_crs, always_xy=True).transform
-        return transform(project, geom)
+        return ops.transform(project, geom)
 
     @staticmethod
     def bounds(
@@ -463,7 +495,8 @@ class MatchCRS:
             raise InvalidInputType("geom", "tuple of length 4", BOX_ORD)
 
         project = pyproj.Transformer.from_crs(in_crs, out_crs, always_xy=True).transform
-        return transform(project, box(*geom)).bounds
+        bbox: Tuple[float, float, float, float] = ops.transform(project, box(*geom)).bounds
+        return bbox
 
     @staticmethod
     def coords(
@@ -552,12 +585,14 @@ def bbox_decompose(
     geod = pyproj.Geod(ellps="WGS84")
     west, south, east, north = _bbox
 
-    def directional_split(az: float, origin: float, dest: float, lvl: float, xy: bool, px: int):
+    def directional_split(
+        az: float, origin: float, dest: float, lvl: float, xy: bool, px: int
+    ) -> Tuple[List[Tuple[float, Any]], List[int]]:
         divs = [0]
         mul = 1.0
         coords = []
 
-        def get_args(dst, dx):
+        def get_args(dst: float, dx: float) -> Tuple[float, float, float, float, int]:
             return (dst, lvl, az, dx, 0) if xy else (lvl, dst, az, dx, 1)
 
         while divs[-1] < 1:
