@@ -2,6 +2,7 @@
 import io
 import sys
 import zipfile
+from datetime import datetime
 
 import pytest
 from shapely.geometry import Polygon
@@ -66,24 +67,44 @@ def test_restful_byid(geometry_nat):
 
 @pytest.mark.flaky(max_runs=3)
 def test_restful_bygeom(geometry_nat):
-    wbd8 = ArcGISRESTful(f"{ServiceURL().restful.wbd}/4")
-    wbd8.n_threads = 4
-    wbd8.oids_bygeom(geometry_nat.bounds)
-    wbd8.oids_bygeom(geometry_nat)
-    huc8_all = wbd8.get_features()
-    wbd8.oids_bygeom(geometry_nat, sql_clause="areasqkm > 5000")
-    huc8_large = wbd8.get_features()
-    assert len(huc8_all[0]["features"]) - len(huc8_large[0]["features"]) == 2
+    geofab = ArcGISRESTful(f"{ServiceURL().restful.nhd_fabric}/1")
+    geofab.n_threads = 4
+    geofab.oids_bygeom(geometry_nat.bounds)
+    geofab.oids_bygeom(geometry_nat)
+    wb_all = geofab.get_features()
+    geofab.oids_bygeom(geometry_nat, sql_clause="areasqkm > 20")
+    wb_large = geofab.get_features()
+    assert len(wb_all[0]["features"]) - len(wb_large[0]["features"]) == 915
+
+
+@pytest.mark.flaky(max_runs=3)
+def test_restful_bymultipoint():
+    url = "https://watersgeo.epa.gov/arcgis/rest/services/NHDPlus/NHDPlus/MapServer"
+    sql_clause = "FTYPE NOT IN (420,428,566)"
+    geom = [
+        (-97.06138, 32.837),
+        (-97.06133, 32.836),
+        (-97.06124, 32.834),
+        (-97.06127, 32.832),
+    ]
+    geo_crs = "epsg:4269"
+    distance = 1500
+
+    service = ArcGISRESTful(url, outformat="json")
+    service.layer = 2  # network flowline
+    service.oids_bygeom(geom, geo_crs=geo_crs, sql_clause=sql_clause, distance=distance)
+    resp = service.get_features(return_m=True)
+    assert len(resp[0]["features"]) == 3
 
 
 @pytest.mark.flaky(max_runs=3)
 def test_restful_bysql():
-    hr = ArcGISRESTful(ServiceURL().restful.nhdplushr, outformat="json")
+    hr = ArcGISRESTful(ServiceURL().restful.nhdplushr_edits, outformat="json")
     hr.layer = 1
     hr.layer = 2
-    hr.oids_bysql("NHDPLUSID IN (5000500013223, 5000400039708, 5000500004825)")
+    hr.oids_bysql("NHDFlowline.PERMANENT_IDENTIFIER IN ('103455178', '103454362', '103453218')")
     resp_sql = hr.get_features(return_m=True)
-    hr.oids_byfield("NHDPLUSID", [5000500013223, 5000400039708, 5000500004825])
+    hr.oids_byfield("NHDFlowline.PERMANENT_IDENTIFIER", ["103455178", "103454362", "103453218"])
     resp_ids = hr.get_features()
 
     assert len(resp_sql[0]["features"]) == len(resp_ids[0]["features"])
@@ -172,35 +193,47 @@ def test_ipv4():
 
 
 @pytest.mark.flaky(max_runs=3)
-def test_async():
-    url = "https://thredds.daac.ornl.gov/thredds/ncss/ornldaac/13281990/daymet_v3_prcp_1990_na.nc4"
-    payload = {
-        "var": "prcp",
-        "north": 45.03690024304768,
-        "west": -68.32580022647919,
-        "east": -67.85159909767978,
-        "south": 44.55340147224697,
-        "disableProjSubset": "on",
-        "horizStride": 1,
-        "time_start": "1990-01-01T12:00:00Z",
-        "time_end": "1990-01-10T12:00:00Z",
-        "timeStride": 1,
-        "addLatLon": "true",
-        "accept": "netcdf",
-    }
-    url_binary = {url: payload}
+def test_async(geometry_nat):
+    west, south, east, north = geometry_nat.bounds
+    base_url = "https://thredds.daac.ornl.gov/thredds/ncss/ornldaac/1299"
+    url_binary = []
+    dates_itr = [(datetime(y, 1, 1), datetime(y, 1, 31)) for y in range(2000, 2005)]
+    url_binary = (
+        (
+            f"{base_url}/MCD13.A{s.year}.unaccum.nc4",
+            {
+                "var": "NDVI",
+                "north": f"{north}",
+                "west": f"{west}",
+                "east": f"{east}",
+                "south": f"{south}",
+                "disableProjSubset": "on",
+                "horizStride": "1",
+                "time_start": s.strftime("%Y-%m-%dT%H:%M:%SZ"),
+                "time_end": e.strftime("%Y-%m-%dT%H:%M:%SZ"),
+                "timeStride": "1",
+                "addLatLon": "true",
+                "accept": "netcdf",
+            },
+        )
+        for s, e in dates_itr
+    )
 
     url_json = [
-        "".join(
-            [
-                "https://labs.waterdata.usgs.gov/api/nldi/linked-data/comid/",
-                "position?coords=POINT%28-68.325%2045.0369%29",
-            ]
+        (
+            "https://labs.waterdata.usgs.gov/api/nldi/linked-data/comid/position",
+            {
+                "f": "json",
+                "coords": "POINT(-68.325 45.0369)",
+            },
         )
     ]
 
     url_text = [
-        "https://waterservices.usgs.gov/nwis/site/?format=rdb&sites=01646500&siteStatus=all"
+        (
+            "https://waterservices.usgs.gov/nwis/site/",
+            {"format": "rdb", "sites": "01646500", "siteStatus": "all"},
+        )
     ]
 
     r_b = pygeoogc.async_requests(url_binary, "binary")
@@ -208,36 +241,14 @@ def test_async():
     r_t = pygeoogc.async_requests(url_text, "text")
 
     assert (
-        sys.getsizeof(r_b[0]) == 187205
+        sys.getsizeof(r_b[0]) == 986161
         and r_j[0]["features"][0]["properties"]["identifier"] == "2675320"
         and r_t[0].split("\n")[-2].split("\t")[1] == "01646500"
     )
 
 
 def test_urls():
-    urls = ServiceURL()
-    assert (
-        urls.restful.nwis == "https://waterservices.usgs.gov/nwis"
-        and urls.restful.nldi == "https://labs.waterdata.usgs.gov/api/nldi"
-        and urls.restful.daymet_point == "https://daymet.ornl.gov/single-pixel/api/data"
-        and urls.restful.daymet_grid == "https://thredds.daac.ornl.gov/thredds/ncss/ornldaac/1328"
-        and urls.restful.wbd == "https://hydro.nationalmap.gov/arcgis/rest/services/wbd/MapServer"
-        and urls.restful.fws == "https://www.fws.gov/wetlands/arcgis/rest/services"
-        and urls.restful.fema
-        == "https://hazards.fema.gov/gis/nfhl/rest/services/public/NFHL/MapServer"
-        and urls.wms.mrlc == "https://www.mrlc.gov/geoserver/mrlc_download/wms"
-        and urls.wms.fema
-        == "https://hazards.fema.gov/gis/nfhl/rest/services/public/NFHLWMS/MapServer/WMSServer"
-        and urls.wms.nm_3dep
-        == "https://elevation.nationalmap.gov/arcgis/services/3DEPElevation/ImageServer/WMSServer"
-        and urls.wms.fws
-        == "https://www.fws.gov/wetlands/arcgis/services/Wetlands_Raster/ImageServer/WMSServer"
-        and urls.wfs.waterdata == "https://labs.waterdata.usgs.gov/geoserver/wmadata/ows"
-        and urls.wfs.fema
-        == "https://hazards.fema.gov/gis/nfhl/services/public/NFHL/MapServer/WFSServer"
-        and urls.http.ssebopeta
-        == "https://edcintl.cr.usgs.gov/downloads/sciweb1/shared/uswem/web/conus/eta/modis_eta/daily/downloads"
-    )
+    assert len(ServiceURL().__dict__["urls"]) == 4
 
 
 def test_show_versions():
